<?php

namespace Drupal\twicpics\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class TwicpicsForm extends ConfigFormBase {

  private $rValidDomain = '/^(?:https?:\/\/)?([^\/]+)\/*$/';
  private $rValidPath = '/^\/*([^\/].+?)\/*$/'; 

  private function trimOrUndefined($value) {
    $trimmed = trim($value);
    return $trimmed !== '' ? $trimmed : null;
  }

  private function parseDomain($value) {
    $domain = $this->trimOrUndefined($value);
    if (!is_null($domain) && preg_match($this->rValidDomain, $domain)) {
      return preg_replace($this->rValidDomain, 'https://$1', $domain);
    } else {
      return "";
    }
  }

  private function parsePath($value) {
    $path = $this->trimOrUndefined($value);
    if (!is_null($path) && preg_match($this->rValidPath, $path)) {
      return preg_replace($this->rValidPath, '/$1/', $path);
    } else {
      return "/";
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'twicpics_admin_settings';
  }


  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'twicpics.admin_settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('twicpics.admin_settings');
    $class = get_class($this);

    $form['twicpics_domain'] = array(
        '#type' => 'textfield',
        '#name' => 'twicpics_domain',
        '#title' => t('TwicPics Domain'),
        '#description' => t('You can find your TwicPic domain in your TwicPics dashboard.'),
        '#size' => 50,
        '#default_value' => $config->get('twicpics_domain'),
        '#required' => TRUE,
        '#element_validate' => [
          function ($element, FormStateInterface $form_state) {
            $value = $form_state->getValue('twicpics_domain');
            $domain = $this->trimOrUndefined($value);
            if (is_null($domain)) {
              $form_state->setError($element, $this->t('Domain is mandatory.'));
            } elseif (!preg_match($this->rValidDomain, $value)) {
              $form_state->setError($element, $this->t('Invalid domain value.'));
            }
          },
        ],
    );

    $form['twicpics_public_path'] = array(
      '#type' => 'textfield',
      '#name' => 'twicpics_public_path',
      '#title' => t('Path for public images'),
      '#size' => 50,
      '#description' => t('This path is configured in the TwicPics back office for the specified domain.'),
      '#default_value' => $config->get('twicpics_public_path'),
      '#required' => FALSE,
    );

    $form['twicpics_private_path'] = array(
      '#type' => 'textfield',
      '#name' => 'twicpics_private_path',
      '#title' => t('Path for private images'),
      '#size' => 50,
      '#description' => t('This path is configured in the TwicPics back office for the specified domain.'),
      '#default_value' => $config->get('twicpics_private_path'),
      '#required' => FALSE,
    );

    $form['twicpics_max_width'] = array(
      '#type' => 'number',
      '#name' => 'twicpics_max_width',
      '#title' => t('TwicPics Max Width (px)'),
      '#size' => 10,
      '#description' => t('Maximum Width of optimized images. This prevent generating large variants.'),
      '#default_value' => $config->get('twicpics_max_width'),
      '#required' => FALSE,
    );

    $form['twicpics_max_height'] = array(
      '#type' => 'number',
      '#name' => 'twicpics_max_height',
      '#title' => t('TwicPics Max Height (px)'),
      '#size' => 10,
      '#description' => t('Maximum Height of optimized images. This prevent generating large variants.'),
      '#default_value' => $config->get('twicpics_max_height'),
      '#required' => FALSE,
    );

    $form['twicpics_api_version'] = array(
      '#type' => 'number',
      '#name' => 'twicpics_api_version',
      '#title' => t('TwicPics API Version'),
      '#size' => 4,
      '#default_value' => null !==($config->get('twicpics_api_version')) ? $config->get('twicpics_api_version') : 1,
      '#step' => 1,
      '#required' => TRUE,
    );

    $form['actions']['#type'] = 'actions';

    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Send'),
      '#button_type' => 'primary'
    );

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $this->config('twicpics.admin_settings')
      ->set('twicpics_domain', $this->parseDomain($form_state->getValue('twicpics_domain')))
      ->set('twicpics_public_path', $this->parsePath($form_state->getValue('twicpics_public_path')))
      ->set('twicpics_private_path', $this->parsePath($form_state->getValue('twicpics_private_path')))
      ->set('twicpics_api_version', $form_state->getValue('twicpics_api_version'))
      ->set('twicpics_max_width', $form_state->getValue('twicpics_max_width'))
      ->set('twicpics_max_height', $form_state->getValue('twicpics_max_height'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
